/* wptMAPI.cpp - MAPI interface for sending keys.
 *	Copyright (C) 2003, 2004, 2005, 2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <stdio.h>
#include <mapi.h>

#include "resource.h"
#include "wptTypes.h"
#include "wptErrors.h"
#include "wptW32API.h"
#include "wptGPG.h"
#include "wptVersion.h"
#include "wptCommonCtl.h"
#include "wptKeyManager.h"


static LPMAPILOGON	    mapi_logon = NULL; 
static LPMAPILOGOFF	    mapi_logoff = NULL;     
static LPMAPISENDDOCUMENTS  mapi_send_documents = NULL; 
static LPMAPISENDMAIL	    mapi_send_mail = NULL;
static HINSTANCE	    hlib = NULL;
static int		    init = 0;

#define load_one_fnc(cast, hlib, name) (cast)GetProcAddress ((hlib), name)


/* Load MAPI library and set function pointers.
   Return value: 0 on success. */
int
mapi_init (void)
{
    if (init)
	return 0;

    hlib = LoadLibrary ("MAPI32.DLL");
    if (!hlib)
	return -1;

    mapi_logon = load_one_fnc (LPMAPILOGON, hlib, "MAPILogon");
    mapi_logoff = load_one_fnc (LPMAPILOGOFF, hlib, "MAPILogoff");
    mapi_send_documents = load_one_fnc (LPMAPISENDDOCUMENTS, hlib, "MAPISendDocuments");
    mapi_send_mail = load_one_fnc (LPMAPISENDMAIL, hlib, "MAPISendMail");
    if (!mapi_logon || !mapi_logoff || !mapi_send_documents || !mapi_send_mail)
	return -1;
    init = 1;

    return 0;
}


/* Free library and cleanup. */
void
mapi_deinit (void)
{
    if (hlib) {
	FreeLibrary (hlib);
	hlib = NULL;
	init = 0;
    }
}


/* Send the file given in @ascfile via the MAPI mechanism. */
int
mapi_send_ascfile (char *ascfile)
{
    LHANDLE hd;
    int rc;

    if (!init)
	return 0;

    rc = mapi_logon (0, NULL, NULL, MAPI_LOGON_UI, 0, &hd);
    if (rc != SUCCESS_SUCCESS) {
	MessageBox (NULL, _("MAPI Login failed."), "MAPI", MB_ICONWARNING|MB_OK);
	goto fail;
    }
    rc = mapi_send_documents (0, ";", ascfile, NULL, 0);
    if (rc == MAPI_E_USER_ABORT)
	rc = SUCCESS_SUCCESS;
    if (rc != SUCCESS_SUCCESS)
	MessageBox (NULL, _("Could not sent mail."), "MAPI", MB_ICONERROR|MB_OK);

fail:
    mapi_logoff (hd, 0, 0, 0);
    return rc;
}


static void
free_mapi_msg (MapiMessage *msg)
{
    if (!msg)
	return;
    safe_free (msg->lpszSubject);
    safe_free (msg->lpszNoteText);
    safe_free (msg);
}


static void
free_recip_tab (MapiRecipDesc *recip, size_t n)
{
    size_t i;

    if (!recip)
	return;
    if (!n)
	return;
    for (i=0; i < n; i++)
	safe_free (recip[i].lpszName);
    safe_free (recip);
}


static void
free_files_tab (MapiFileDesc *files, size_t n)
{
    size_t i;

    if (!files)
	return;
    if (!n)
	return;
    for (i=0; i < n; i++) {
	safe_free (files[i].lpszFileName);
	safe_free (files[i].lpszPathName);
    }
    safe_free (files);
}


/* Same as mapi_send_pubkey but there is an additional note. */
int
mapi_send_pubkey_ext (gpgme_key_t to, const char *keyfile, int flags)
{
    LHANDLE hd;
    MapiMessage *msg;
    MapiRecipDesc *recip;
    MapiFileDesc *attch;
    char *p, *kinf;
    const char *s;
    int rc;

    if (!init)
	return 0;

    rc = mapi_logon (0, NULL, NULL, MAPI_LOGON_UI, 0, &hd);
    if (rc != SUCCESS_SUCCESS) {
	MessageBox (NULL, "MAPI Login failed.", "MAPI", MB_ICONWARNING|MB_OK);
	return rc;
    }

    msg = (MapiMessage *)calloc (1, sizeof *msg);
    if (!msg)
	BUG (0);
    p = msg->lpszSubject = strdup ("OpenPGP Public Key");
    if (!p)
	BUG (0);

    s = "Attached is this OpenPGP public key:\n%s\n\n"
	"Import this key via the clipboard or the Key Manager to\n"
        "exchange encrypted mails with the key holder and to be able\n"
        "to verify its signatures.\n"
        "\n"
        "If you don't have WinPT, you can download it at http://www.winpt.org";
    kinf = km_key_get_info (to, 0);
    p = (char*)malloc (strlen (s) + strlen (kinf) + 2);
    sprintf (p, s, kinf);
    free_if_alloc (kinf);

    p = msg->lpszNoteText = p;
    if (!p)
	BUG (0); 

    /* If the key was signed, we assume it shall be sent back to the owner. */
    if (flags) {
	recip = (MapiRecipDesc *)calloc (1, sizeof *recip);
	if (!recip)
	    BUG (0);
	recip[0].ulRecipClass = MAPI_TO;
	p = recip[0].lpszName = strdup (to->uids->uid);	
	if (!p)    
	    BUG (0);
	msg->lpRecips = recip;
	msg->nRecipCount = 1;
    }
    else {
	msg->lpRecips = recip = NULL;
	msg->nRecipCount = 0;
    }

    msg->nFileCount = 1;
    attch = (MapiFileDesc *)calloc (1, sizeof *attch);
    if (!attch)
	BUG (0);
    attch[0].lpszFileName = strdup (keyfile);
    attch[0].lpszPathName = strdup (keyfile);
    msg->lpFiles = attch;

    rc = mapi_send_mail (hd, 0, msg, MAPI_DIALOG , 0);
    if (rc == MAPI_E_USER_ABORT)
	rc = SUCCESS_SUCCESS;
    if (rc != SUCCESS_SUCCESS)
	MessageBox (NULL, _("Could not sent mail."), "MAPI", MB_ERR);

    free_recip_tab (recip, 1);
    free_files_tab (attch, 1);
    free_mapi_msg (msg);
    mapi_logoff (hd, 0, 0, 0);

    return 0;
}


#if 0
/* Send a public key stored in @keyfile with the keyid @keyid
   via the MAPI mechanism to a mail recipient. 
   Return value: SUCCESS_SUCCESS on succes. */
int
mapi_send_pubkey2 (const char *keyid, char *keyfile)
{
    LHANDLE hd;
    const char * fmt;
    char * keyinf = NULL;
    int rc;

    if (!init)
	return 0;

    fmt = _("GPG Public Key of %s");
    keyinf = new char[strlen (fmt) + strlen (keyid) + 2];
    if (!keyinf)
	BUG (0);
    sprintf (keyinf, fmt, keyid);
    rc = mapi_logon (0, NULL, NULL, MAPI_LOGON_UI, 0, &hd);
    if (rc != SUCCESS_SUCCESS) {
	MessageBox (NULL, _("MAPI Login failed."), "MAPI", MB_ICONWARNING|MB_OK);
	goto fail;
    }
    rc = mapi_send_documents (0, ";", keyfile, keyinf, 0);
    if (rc == MAPI_E_USER_ABORT)
	rc = SUCCESS_SUCCESS;
    if (rc != SUCCESS_SUCCESS)
	MessageBox (NULL, _("Could not sent mail."), "MAPI", MB_ICONERROR|MB_OK);

fail:
    mapi_logoff (hd, 0, 0, 0);
    free_if_alloc (keyinf);
    return rc;
}
#endif


int
mapi_send_pubkey (const char *keyid, char *keyfile)
{
    gpgme_key_t key;

    get_pubkey (keyid, &key);
    return mapi_send_pubkey_ext (key, keyfile, 0);
}
